@@ -1,7 +1,7 @@
 use strict;
 use warnings;
 package Net::SAML2::Protocol::AuthnRequest;
-our $VERSION = '0.61'; # VERSION
+our $VERSION = '0.62'; # VERSION
 
 use Moose;
 use MooseX::Types::URI qw/ Uri /;
@@ -84,6 +84,20 @@ has 'RequestedAuthnContext_Comparison' => (
     default => 'exact'
 );
 
+has 'force_authn' => (
+    isa     => 'Bool',
+    is      => 'ro',
+    required => 0,
+    predicate => 'has_force_authn',
+);
+
+has 'is_passive' => (
+    isa     => 'Bool',
+    is      => 'ro',
+    required => 0,
+    predicate => 'has_is_passive',
+);
+
 around BUILDARGS => sub {
     my $orig = shift;
     my $self = shift;
@@ -133,12 +147,15 @@ sub as_xml {
         'destination'          => 'Destination',
         'issuer_namequalifier' => 'NameQualifier',
         'issuer_format'        => 'Format',
+        'force_authn'          => 'ForceAuthn',
+        'is_passive'           => 'IsPassive',
     );
 
     my @opts = qw(
         assertion_url assertion_index protocol_binding
         attribute_index provider_name destination
-        issuer_namequalifier issuer_format
+        issuer_namequalifier issuer_format force_authn
+        is_passive
     );
 
     foreach my $opt (@opts) {
@@ -152,6 +169,9 @@ sub as_xml {
         elsif (any { $opt eq $_ } qw(issuer_namequalifier issuer_format)) {
             $issuer_attrs{ $att_map{$opt} } = $val;
         }
+        elsif (any { $opt eq $_ } qw(force_authn is_passive)) {
+            $req_atts{ $att_map{$opt} } = ( $val ? 'true' : 'false' );
+        }
         else {
             $req_atts{ $att_map{$opt} } = $val;
         }
@@ -226,7 +246,7 @@ Net::SAML2::Protocol::AuthnRequest - SAML2 AuthnRequest object
 
 =head1 VERSION
 
-version 0.61
+version 0.62
 
 =head1 SYNOPSIS
 
@@ -236,6 +256,8 @@ version 0.61
     destination   => $destination,	# Identity Provider (IdP) SSO URL
     provider_name => $provider_name,	# Service Provider (SP) Human Readable Name
     issue_instant => DateTime->now,	# Defaults to Current Time
+    force_authn   => $force_authn,	# Force new authentication (Default: false)
+    is_passive    => $is_passive,	# IdP should not take control of UI (Default: false)
   );
 
   my $request_id = $authnreq->id;	# Store and Compare to InResponseTo
@@ -250,6 +272,8 @@ version 0.61
     destination   => $destination,	# Identity Provider (IdP) SSO URL
     provider_name => $provider_name,	# Service Provider (SP) Human Readable Name
     issue_instant => DateTime->now,	# Defaults to Current Time
+    force_authn   => $force_authn,	# Force new authentication (Default: false)
+    is_passive    => $is_passive,	# IdP should not take control of UI (Default: false)
   );
 
 =head1 NAME
